#ifndef RBTREE_H_
#define RBTREE_H_

#include <stddef.h>
#include <stdbool.h>

struct rbtree;
struct rbtree_iterator;

struct rbtree * rbtree_new(bool (*le)(void * a, void * b), bool (*eq)(void * a, void * b));
void rbtree_delete(struct rbtree * rbtree);

void * rbtree_insert (struct rbtree * rbtree, void * data);
void * rbtree_search (struct rbtree * rbtree, void * data);
void * rbtree_search_iterative (struct rbtree * rbtree, void * data);
void * rbtree_minimum (struct rbtree * rbtree);
void * rbtree_maximum (struct rbtree * rbtree);
unsigned int rbtree_size (struct rbtree * rbtree);

void rbtree_walk_inorder   (struct rbtree * rbtree, void (*op)(void * data));
void rbtree_walk_preorder  (struct rbtree * rbtree, void (*op)(void * data));
void rbtree_walk_postorder (struct rbtree * rbtree, void (*op)(void * data));
void rbtree_walk_breadth_first (struct rbtree * rbtree, void (*op)(void * data));

struct rbtree_iterator * rbtree_begin  (struct rbtree * rbtree);
struct rbtree_iterator * rbtree_end    (struct rbtree * rbtree);
struct rbtree_iterator * rbtree_next   (struct rbtree_iterator * it);
struct rbtree_iterator * rbtree_prev   (struct rbtree_iterator * it);
void *                   rbtree_find   (struct rbtree_iterator * it, void * data);
void *                   rbtree_remove (struct rbtree_iterator * it);
void *                   rbtree_value  (struct rbtree_iterator * it);

unsigned int rbtree_black_depth(struct rbtree * rbtree);
void rbtree_dump(struct rbtree * rbtree, void (*print_val)(void *) );

#endif
